<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2009 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_3/pages/pubmarketbuy.php $
 * $Id: pubmarketbuy.php 1692 2009-11-01 20:48:01Z Quietust $
 */

if (!defined('IN_GAME'))
	die("Access denied");

$title = 'PUBMARKETBUY_TITLE';

page_header();

if (ROUND_FINISHED)
	unavailable(lang('PUBMARKETBUY_UNAVAILABLE_END'));
if (!ROUND_STARTED)
	unavailable(lang('PUBMARKETBUY_UNAVAILABLE_START'));
if ($emp1->is_protected())
	unavailable(lang('PUBMARKETBUY_UNAVAILABLE_PROTECT'));
if ($emp1->e_flags & EFLAG_ADMIN)
	unavailable(lang('PUBMARKETBUY_UNAVAILABLE_ADMIN'));

$types = lookup('list_mkt');
$stores = lookup('pubmkt_name_id');

// Determines the current buying price of a unit
function getCost ($emp, $type, &$amount, &$price)
{
	global $db, $stores;
	$q = $db->prepare('SELECT SUM(k_amt) AS k_amt,k_price FROM '. MARKET_TABLE .' WHERE k_type = ? AND e_id != ? AND k_time <= ? GROUP BY k_price HAVING k_price = MIN(k_price)');
	$q->bindIntValue(1, $stores[$type]);
	$q->bindIntValue(2, $emp->e_id);
	$q->bindIntValue(3, CUR_TIME);
	$q->execute() or warning('Failed to get market amounts and prices', 0);
	if ($row = $q->fetch())
	{
		$amount = $row['k_amt'];
		$price = $row['k_price'];
	}
	else	$amount = $price = 0;
}

if ($action == 'buy') do
{
	if ($db->queryCell('SELECT COUNT(*) FROM '. MARKET_TABLE .' WHERE lock_id = ?', array($user1->u_id)) > 0)
	{
		notice(lang('PUBMARKETBUY_IMPOSSIBLE_LOCKED'));
		break;
	}
	foreach ($types as $type)
	{
		$amount = fixInputNum(getFormVar('buy_'. $type));
		if ($amount == 0)
			continue;
		$price = fixInputNum(getFormVar('price_'. $type));
		// require that they have enough money to buy what they wanted at the price they saw
		// if prices go down, well, lucky them - they won't have to pay as much
		if ($amount * $price > $emp1->e_cash)
		{
			notice(lang('PUBMARKETBUY_NOT_ENOUGH_MONEY', $emp1->era->getData($type)));
			continue;
		}

		$bought = array();	// how many units were bought at each price
		$unbought = 0;		// in case we ask for more than we can get
		$totalspent = 0;

		// other people may be buying items, so we need to work around locks
		while ($amount > 0)
		{
			// delay 0.1 to 0.2 seconds between each check, so as to not overload the server
			usleep(mt_rand(100000, 200000));

			// See how much is available on the market at the specified price
			$max = $db->queryCell('SELECT SUM(k_amt) FROM '. MARKET_TABLE .' WHERE k_type = ? AND k_price <= ? AND e_id != ? AND k_time <= ?', array($stores[$type], $price, $emp1->e_id, CUR_TIME));
			// if there isn't enough, lower the desired purchase amount
			if ($max < $amount)
			{
				$unbought += $amount - $max;
				$amount = $max;
				continue;	// loop again - it might have gone down to zero
			}

			// Lock all market entries which satisfy the criteria
			$q = $db->prepare('UPDATE '. MARKET_TABLE .' SET lock_id = ? WHERE k_type = ? AND k_price <= ? AND e_id != ? AND k_time <= ? AND k_amt > 0 AND lock_id = 0');
			$q->bindIntValue(1, $user1->u_id);
			$q->bindIntValue(2, $stores[$type]);
			$q->bindIntValue(3, $price);
			$q->bindIntValue(4, $emp1->e_id);
			$q->bindIntValue(5, CUR_TIME);
			$q->execute() or warning('Failed to lock market items', 0);

			// then fetch all locked rows
			$q = $db->prepare('SELECT k_id,e_id,k_amt,k_price FROM '. MARKET_TABLE .' WHERE lock_id = ? ORDER BY k_price ASC, k_time ASC');
			$q->bindIntValue(1, $user1->u_id);
			$q->execute() or warning('Failed to fetch locked market items', 0);
			$items = $q->fetchAll();

			$upd = $db->prepare('UPDATE '. MARKET_TABLE .' SET k_amt = ? WHERE k_id = ?');
			$del = $db->prepare('DELETE FROM '. MARKET_TABLE .' WHERE k_id = ?');
			foreach ($items as $item)
			{
				$buyamt = min($item['k_amt'], $amount);

				if (!isset($bought[$item['k_price']]))
					$bought[$item['k_price']] = 0;
				$bought[$item['k_price']] += $buyamt;

				$spent = $buyamt * $item['k_price'];
				$taxed = round($spent * 0.05);
				$totalspent += $spent;

				$emp1->addData('e_'. $type, $buyamt);
				$emp1->e_cash -= $spent;
				if ($item['k_amt'] == $buyamt)
				{
					$del->bindIntValue(1, $item['k_id']);
					$del->execute() or warning('Failed to remove bought items from market', 0);
				}
				else
				{
					$upd->bindIntValue(1, $item['k_amt'] - $buyamt);
					$upd->bindIntValue(2, $item['k_id']);
					$upd->execute() or warning('Failed to update bought items on market', 0);
				}

				$emp_a = prom_empire::cached_load($item['e_id']);
				addEmpireNews(EMPNEWS_ATTACH_MARKET_SELL, $emp1, $emp_a, $stores[$type], $buyamt, $spent, $spent - $taxed);
				$emp_a = NULL;

				// Public Market taxes go into the lottery
				$world->adjust('lotto_current_jackpot', $taxed);

				$amount -= $buyamt;
				if ($amount == 0)
					break;
			}

			// finally, unlock all locked rows
			$q = $db->prepare('UPDATE '. MARKET_TABLE .' SET lock_id = 0 WHERE lock_id = ?');
			$q->bindIntValue(1, $user1->u_id);
			$q->execute() or warning('Failed to unlock items on market', 0);
		}
		if (array_sum($bought) > 0)
		{
			if (min(array_keys($bought)) < $price)
				notice(lang('PUBMARKETBUY_CHEAPER_ARRIVED', $emp1->era->getData($type)));
			$buys = array();
			foreach ($bought as $cost => $amt)
				$buys[] = lang('PUBMARKETBUY_BOUGHT_LINE', number($amt), $emp1->era->getData($type), money($cost));
			if (count($bought) == 1)
				notice(lang('PUBMARKETBUY_BOUGHT_SINGLE', commalist($buys), money($totalspent)));
			else	notice(lang('PUBMARKETBUY_BOUGHT_MULTIPLE', commalist($buys), number(array_sum($bought)), $emp1->era->getData($type), money($totalspent)));
			if ($unbought > 0)
				notice(lang('PUBMARKETBUY_BOUGHT_NOT_ALL', number($unbought), $emp1->era->getData($type)));
		}
		else	notice(lang('PUBMARKETBUY_BOUGHT_NONE', number($unbought), $emp1->era->getData($type), money($price)));
		logevent(varlist(array('type', 'price', 'bought', 'totalspent', 'unbought'), get_defined_vars()));
	}
} while (0);
notices();

$costs = array();
$amounts = array();
foreach ($types as $type)
	getCost($emp1, $type, $amounts[$type], $costs[$type]);
?>
<form method="post" action="?location=pubmarketbuy">
<table class="inputtable">
<tr><td colspan="3"><a href="?location=pubmarketbuy"><?=lang('PUBMARKET_LINK_BUY')?></a></td>
    <td colspan="3" class="ar"><a href="?location=pubmarketsell"><?=lang('PUBMARKET_LINK_SELL')?></a></td></tr>
<tr><th class="al"><?=lang('COLUMN_UNIT')?></th>
    <th class="ar"><?=lang('COLUMN_OWNED')?></th>
    <th class="ar"><?=lang('COLUMN_AVAIL')?></th>
    <th class="ar"><?=lang('COLUMN_PRICE')?></th>
    <th class="ar"><?=lang('COLUMN_CANBUY')?></th>
    <th class="ar"><?=lang('COLUMN_BUY')?></th></tr>
<?
foreach ($types as $type)
{
?>
<tr><td><?=lang($emp1->era->getData($type))?></td>
    <td class="ar"><?=number($emp1->getData('e_'. $type))?></td>
    <td class="ar"><?=number($amounts[$type])?></td>
    <td class="ar"><input type="hidden" name="price_<?=$type?>" value="<?=$costs[$type]?>" /><?=money($costs[$type])?></td>
    <td class="ar"><?=number(min(floor($emp1->e_cash / max($costs[$type], 1)), $amounts[$type]))?></td>
    <td class="ar"><input type="text" name="buy_<?=$type?>" size="6" value="0" /></td></tr>
<?
}
?>
<tr><td colspan="6" class="ac"><input type="hidden" name="action" value="buy" /><input type="submit" value="<?=lang('PUBMARKETBUY_SUBMIT')?>" /></td></tr>
</table>
</form>
<?
page_footer();
?>
