<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2009 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_3/pages/signup.php $
 * $Id: signup.php 1692 2009-11-01 20:48:01Z Quietust $
 */

if (!defined('IN_GAME'))
	die('Access denied');

if (!ROUND_SIGNUP)
	error_403('ERROR_TITLE', lang('ERROR_SIGNUP_CLOSED'));

if (SIGNUP_CLOSED_USER && SIGNUP_CLOSED_EMPIRE)
	error_403('ERROR_TITLE', lang('ERROR_SIGNUP_DISABLED'));

$username = getFormVar('signup_username');
$password = getFormVar('signup_password');
$password_verify = getFormVar('signup_password_verify');
$name = htmlspecialchars(getFormVar('signup_name'));
$email = getFormVar('signup_email');
$email_verify = getFormVar('signup_email_verify');
$language = getFormVar('signup_lang');
$empname = htmlspecialchars(getFormVar('signup_empirename'));
$race = getFormVar('signup_race', RACE_HUMAN);

if (!setlanguage($language))
	$language = $cur_lang;

$races = prom_race::getNames();

// bounced from the login page, so prefill the username
$reg = fixInputBool(getFormVar('registered'));
if ($reg && !$username)
	$username = $reg;

$html = new prom_html_compact();

if ($action == 'signup') do
{
	// Part 1 - Check if the username and password are specified and legal
	if (empty($username))
	{
		notice(lang('INPUT_NEED_USERNAME'));
		break;
	}
	if (empty($password))
	{
		notice(lang('INPUT_NEED_PASSWORD'));
		break;
	}
	if (strlen($username) > 255)
	{
		notice(lang('INPUT_USERNAME_TOO_LONG'));
		break;
	}
	if (lang_isset($username))
	{
		notice(lang('INPUT_BAD_USERNAME'));
		break;
	}

	// Part 2 - Username and password are legal, now see if the user can actually be created
	$user = new prom_user();
	if ($user->findName($username))
	{
		if (SIGNUP_CLOSED_EMPIRE)
		{
			notice(lang('SIGNUP_EMPIRE_CLOSED', 'SIGNUP_HEADER_EXTRA'));
			break;
		}
		// The user has registered here once before, so let them create a new empire with it
		$user->load();
		if (!$user->checkPassword($password))
		{
			// If they typed the password twice, they were probably trying to create a new one
			// otherwise, they likely just typed it wrong
			if (strlen($password_verify))
				notice(lang('INPUT_USERNAME_IN_USE'));
			else
			{
				notice(lang('INPUT_INCORRECT_PASSWORD'));
				logmsg(E_USER_NOTICE, 'failed (password) - '. $username);
			}
			break;
		}
		if ($user->u_flags & UFLAG_DISABLE)
		{
			notice(lang('SIGNUP_ACCOUNT_DISABLED'));
			logmsg(E_USER_NOTICE, 'failed (disabled) - '. $username);
			break;
		}
		if ($user->u_flags & UFLAG_CLOSED)
		{
			notice(lang('SIGNUP_ACCOUNT_CLOSED'));
			logmsg(E_USER_NOTICE, 'failed (closed) - '. $username);
			break;
		}
		$lock['user1'] = $user->u_id;
		prom_entity::lock_entities(array($user), $user->u_id);	// user exists, password OK, so lock it
	}
	else
	{
		if (SIGNUP_CLOSED_USER)
		{
			notice(lang('SIGNUP_USER_CLOSED', 'SIGNUP_HEADER_EXTRA'));
			break;
		}
		// Try to create a new account for the user
		if (empty($name))
		{
			notice(lang('INPUT_NEED_NICKNAME'));
			break;
		}
		if (strlen($name) > 255)
		{
			notice(lang('INPUT_NICKNAME_TOO_LONG'));
			break;
		}
		if (lang_isset($name))
		{
			notice(lang('INPUT_BAD_NICKNAME'));
			break;
		}
		if (!validate_email($email))
		{
			notice(lang('INPUT_NEED_EMAIL'));
			break;
		}
		if (strlen($email) > 255)
		{
			notice(lang('INPUT_EMAIL_TOO_LONG'));
			break;
		}
		if ($ban = check_banned_email($email))
		{
			notice(lang('SIGNUP_EMAIL_BANNED', ($ban['p_reason']) ? $ban['p_reason'] : lang('BANNED_NO_REASON')));
			break;
		}
		if ($password != $password_verify)
		{
			notice(lang('INPUT_PASSWORD_MISMATCH'));
			break;
		}
		if ($email != $email_verify)
		{
			notice(lang('INPUT_EMAIL_MISMATCH'));
			break;
		}
		if ($db->queryCell('SELECT COUNT(*) FROM '. USER_TABLE .' WHERE u_email = ?', array($email)) > 0)
		{
			notice(lang('INPUT_EMAIL_IN_USE'));
			break;
		}
		$user->create($username, $password, $name, $email, $language);
		$user->save(FALSE);
		$lock['user1'] = $user->u_id;
		logevent(varlist(array('username', 'name', 'email'), get_defined_vars()));
		if (SIGNUP_CLOSED_EMPIRE)
		{
			$user->unlock();
?>
<br /><?=lang('SIGNUP_CANNOT_CONTINUE')?><br /><br />
<?
			$html->end();
			exit;
		}
	}

	// Part 3 - Attempt to create an empire for the user
	// At this point, the user record is locked, so we MUST unlock it if we encounter any problems below

	$accounts = $db->queryCell('SELECT COUNT(*) FROM '. EMPIRE_TABLE .' WHERE u_id = ?', array($user->u_id)) + 1;

	if (empty($empname))
	{
		notice(lang('INPUT_NEED_EMPIRE'));
		$user->unlock();
		break;
	}
	if (empty($race))
	{
		notice(lang('SIGNUP_NEED_RACE'));
		$user->unlock();
		break;
	}
	if (strlen($empname) > 255)
	{
		notice(lang('INPUT_EMPIRE_TOO_LONG'));
		$user->unlock();
		break;
	}
	if (lang_isset($empname))
	{
		notice(lang('INPUT_BAD_EMPIRE'));
		break;
	}
	if (!prom_race::exists($race))
	{
		notice(lang('SIGNUP_RACE_INVALID'));
		$user->unlock();
		break;
	}
	if ($db->queryCell('SELECT COUNT(*) FROM '. EMPIRE_TABLE .' WHERE e_name = ?', array($empname)) > 0)
	{
		notice(lang('INPUT_EMPIRE_IN_USE'));
		$user->unlock();
		break;
	}
	if ($accounts > EMPIRES_PER_USER)
	{
		notice(lang('SIGNUP_NO_MORE_EMPIRES'));
		$user->unlock();
		break;
	}

	// Everything is good - proceed and create the empire

	$emp = new prom_empire();
	$emp->create($user, $empname, $race);
	$lock['emp1'] = $emp->e_id;

	// establish login session with this empire selected
	prom_session::start();
	$_SESSION['user'] = $user->u_id;
	$_SESSION['empire'] = $emp->e_id;

	$user->u_lastip = $_SERVER['REMOTE_ADDR'];
	$user->u_lastdate = CUR_TIME;
	$emp->setFlag(EFLAG_ONLINE);

	// commit to database
	$user->save();
	$emp->save();

	$html->begin('SIGNUP_TITLE');
	echo lang('SIGNUP_COMPLETE', $emp, $user->u_email) .'<br />';
	if ($accounts > 1)
		echo lang('SIGNUP_MULTIPLE') .'<br />';
?>
<br /><a href="?location=game"><?=lang('SIGNUP_CONTINUE')?></a><br /><br />
<?
	$mailerror = $emp->sendValidationMail($user);
	if ($mailerror)
		echo '<div class="cwarn">'. lang('SIGNUP_MAILERROR', $mailerror) .'</div>';
	$html->end();
	logevent(varlist(array('username', 'empname', 'race', 'accounts', 'mailerror'), get_defined_vars()));
	exit;
} while (0);
$html->begin('SIGNUP_TITLE');
?>
<h2><?=lang('SIGNUP_HEADER')?></h2>
<?
if ($reg)
	echo lang('SIGNUP_WELCOME_BACK', GAME_TITLE) .'<br />';
else	echo lang('SIGNUP_WELCOME_FIRST', GAME_TITLE, GRAVEYARD_DISCLOSE ? 'SIGNUP_WELCOME_DISCLOSE' : 'SIGNUP_WELCOME_NO_DISCLOSE') .'<br />';
?>
<b><?=lang('SIGNUP_WARN_RULES')?></b><br /><br />
<table class="inputtable">
<caption style="font-size:large;font-weight:bold"><?=lang('SIGNUP_HEADER_RULES')?></caption>
<tr><th style="font-size:large"><?=lang('SIGNUP_HEADER_MULTIS')?></th></tr>
<tr><td class="ac"><?=lang('SIGNUP_RULES_MULTIS', plural(EMPIRES_PER_USER, 'SIGNUP_MULTIS_SINGLE', 'SIGNUP_MULTIS_PLURAL'))?></td></tr>
<tr><th style="font-size:large"><?=lang('SIGNUP_HEADER_USE')?></th></tr>
<tr><td class="ac"><?=lang('SIGNUP_RULES_USE')?></td></tr>
<tr><th style="font-size:large"><?=lang('SIGNUP_HEADER_SUPPORT')?></th></tr>
<tr><td class="ac"><?=lang('SIGNUP_RULES_SUPPORT', GAME_TITLE, MAIL_ADMIN)?></td></tr>
<tr><th style="font-size:large"><?=lang('SIGNUP_HEADER_EXTRA')?></th></tr>
<tr><td class="ac"><?=TXT_RULES?></td></tr>
</table><br />
<h3><?=lang('SIGNUP_VALIDATION_REMINDER', MAIL_VALIDATE)?></h3>
<?
notices(2);
?>
<h4><?if (SIGNUP_CLOSED_USER) echo lang('SIGNUP_USER_CLOSED_REMINDER', 'SIGNUP_HEADER_EXTRA'); elseif (SIGNUP_CLOSED_EMPIRE) lang('SIGNUP_EMPIRE_CLOSED_REMINDER', 'SIGNUP_HEADER_EXTRA'); else echo lang('SIGNUP_REPLAY_REMINDER');?></h4>
<form method="post" action="?location=signup">
<table class="inputtable">
<tr><th colspan="2"><?=lang('SIGNUP_ACCOUNT_INFO')?></th></tr>
<tr><th class="ar"><?=lang('LABEL_USERNAME')?></th>
    <td><input type="text" name="signup_username" value="<?=htmlspecialchars($username)?>" size="8" maxlength="24" /></td></tr>
<tr><th class="ar"><?=lang('LABEL_PASSWORD')?></th>
    <td><input type="password" name="signup_password" size="8" /></td></tr>
<?
if (!SIGNUP_CLOSED_USER)
{
?>
<tr><th class="ar">* <?=lang('LABEL_PASSWORD_VERIFY')?></th>
    <td><input type="password" name="signup_password_verify" size="8" /></td></tr>
<tr><th class="ar">* <?=lang('LABEL_NICKNAME')?></th>
    <td><input type="text" name="signup_name" value="<?=$name?>" size="24" /></td></tr>
<tr><th class="ar">* <?=lang('LABEL_EMAIL')?></th>
    <td><input type="text" name="signup_email" value="<?=htmlspecialchars($email)?>" size="24" /></td></tr>
<tr><th class="ar">* <?=lang('LABEL_EMAIL_VERIFY')?></th>
    <td><input type="text" name="signup_email_verify" size="24" /></td></tr>
<tr><th class="ar">* <?=lang('SIGNUP_LANGUAGE')?></th>
    <td><?
$langlist = array();
foreach (array_keys($lang) as $lang_id)
	$langlist[$lang_id] = $lang[$lang_id]['LANG_ID'];
echo optionlist('signup_lang', $langlist, $language);
?></td></tr>
<tr><td colspan="2" style="font-size:small;text-align:center"><?=lang('SIGNUP_PRIVACY')?></td></tr>
<?
}
?>
<?
if (!SIGNUP_CLOSED_EMPIRE)
{
?>
<tr><th colspan="2"><?=lang('SIGNUP_EMPIRE_INFO')?></th></tr>
<tr><th class="ar"><?=lang('LABEL_EMPIRE')?></th>
    <td><input type="text" name="signup_empirename" value="<?=$empname?>" size="24" maxlength="32" /></td></tr>
<tr><th class="ar"><?=lang('LABEL_RACE')?></th>
    <td><?
$racelist = array();
foreach ($races as $rid => $rname)
	$racelist[$rid] = $rname;
echo optionlist('signup_race', $racelist, $race);
?> <a href="?location=pguide&amp;section=races"><?=lang('SIGNUP_RACE_DETAILS')?></a></td></tr>
<?
}
?>
<tr><td colspan="2" class="ac"><input type="hidden" name="action" value="signup" /><input type="submit" value="<?=lang('SIGNUP_SUBMIT')?>" /></td></tr>
</table>
</form>
<?
$html->end();
?>
