<?
/* QM Promisance - Turn-based strategy game
 * Copyright (C) 2001-2009 QMT Productions
 *
 * $URL: https://qmpromisance.svn.sourceforge.net/svnroot/qmpromisance/promisance/tags/release-4_3/turns.php $
 * $Id: turns.php 1692 2009-11-01 20:48:01Z Quietust $
 */

error_reporting(E_ALL | E_STRICT);
date_default_timezone_set('UTC');
ignore_user_abort(1);
set_time_limit(0);

// require CLI invocation
if (isset($_SERVER['REQUEST_URI']))
	die("Access denied");

// prevent recursion
if (defined('IN_GAME'))
	die("Access denied");

define('IN_GAME', TRUE);
define('IN_TURNS', TRUE);
define('PROM_BASEDIR', dirname(__FILE__) . '/');

// Don't allow accessing anything while setup.php is being run
if (file_exists(PROM_BASEDIR .'setup.php'))
	die("Access denied");

require_once(PROM_BASEDIR .'config.php');
require_once(PROM_BASEDIR .'includes/constants.php');
require_once(PROM_BASEDIR .'includes/database.php');
require_once(PROM_BASEDIR .'includes/language.php');
require_once(PROM_BASEDIR .'includes/logging.php');
require_once(PROM_BASEDIR .'includes/misc.php');
require_once(PROM_BASEDIR .'includes/news.php');
require_once(PROM_BASEDIR .'classes/prom_vars.php');
require_once(PROM_BASEDIR .'classes/prom_user.php');
require_once(PROM_BASEDIR .'classes/prom_empire.php');
require_once(PROM_BASEDIR .'classes/prom_clan.php');

// these should be sampled when the script first starts (skipping DB auth time)
// but the rest should be dynamic so we can see how long it takes each part to run
$hour = gmdate('H', CUR_TIME);
$min = gmdate('i', CUR_TIME);

function statecho ($message)
{
	global $hour, $min;
	$time = explode(' ', microtime());
	echo sprintf('%s.%06d - [%02d:%02d] - ', date('Y/m/d H:i:s', $time[1]), $time[0] * 1000000, $hour, $min) . $message ."\n";
}

function lock_all ()
{
	global $world, $db;
	statecho('Locking world variables');
	// Use standard locking on world variables; this way, if the turns script gets queued up
	// multiple times, they won't all rush through at the same time after the first is done
	// Delay 50-100ms before each lock retry
	while (1)
	{
		$worldlock = $world->lock(LOCK_TURNS);
		if ($worldlock)
			break;
		usleep(mt_rand(50000, 100000));
	}
	statecho('Locking users');
	prom_user::lock_all(LOCK_TURNS);

	statecho('Locking empires');
	prom_empire::lock_all(LOCK_TURNS);

	if (CLAN_ENABLE)
	{
		statecho('Locking clans');
		prom_clan::lock_all(LOCK_TURNS);
	}
	statecho('Lock complete');
}

function unlock_all ()
{
	global $world, $db;

	// purge all sessions older than 2 hours - may as well do this here
	statecho('Purging stale sessions');
	$db->queryParam('DELETE FROM '. SESSION_TABLE .' WHERE sess_time < ?', array(CUR_TIME - 2 * 60 * 60));

	// save world variables first, but keep them locked
	statecho('Saving world variables');
	$world->save(FALSE);
	statecho('Unlocking entities');
	// then remove all locks at once
	$db->queryParam('DELETE FROM '. LOCK_TABLE .' WHERE lock_owner = ?', array(LOCK_TURNS));
	statecho('Unlock complete');
}

function done ()
{
	global $starttime;
	$duration = microtime(TRUE) - $starttime;
	echo "Total time elapsed: $duration seconds\n";
	echo "\n";
	exit;
}

$starttime = microtime(TRUE);
statecho('Beginning turn run');

$db = db_open(DB_TYPE, DB_SOCK, DB_HOST, DB_PORT, DB_USER, DB_PASS, DB_NAME);
if (!$db)
{
	statecho('Database is offline - aborting');
	done();
}

$world = new prom_vars();
$world->load();
if (!isset($world->round_time_begin))
{
	statecho('Game setup is not yet complete');
	done();
}
lock_all();

// if round hasn't started, update idle times so empires don't expire
if (CUR_TIME < $world->round_time_begin)
{
	statecho('Round has not yet started - updating idle times');
	$db->queryParam('UPDATE '. EMPIRE_TABLE .' SET e_idle = ?', array(CUR_TIME));
	// During initial signup, don't leave people marked as online
	$db->queryParam('UPDATE '. EMPIRE_TABLE .' SET e_flags = e_flags & ?', array(~EFLAG_ONLINE));
	unlock_all();
	done();
}

// if round has already ended, just exit
if (CUR_TIME >= $world->round_time_end)
{
	statecho('Round has ended - nothing to do');
	// Once the round is over, clear people's online flags
	$db->queryParam('UPDATE '. EMPIRE_TABLE .' SET e_flags = e_flags & ?', array(~EFLAG_ONLINE));
	unlock_all();
	done();
}

statecho('Adjusting world variables');
$q = $db->query('SELECT * FROM '. VAR_ADJUST_TABLE);
$adjusts = $q->fetchAll();
foreach ($adjusts as $adjust)
	$world->setData($adjust['v_name'], $world->getData($adjust['v_name']) + $adjust['v_offset']);
$db->query('DELETE FROM '. VAR_ADJUST_TABLE);

if ($min == TURNS_OFFSET)
{
	if ($hour == 12)
	{
		statecho('Performing daily events');
		$numplayers = $db->queryCell('SELECT COUNT(*) FROM '. EMPIRE_TABLE);
		$lotterynum = mt_rand(1, LOTTERY_MAXTICKETS * $numplayers);

		$jackpot = $world->lotto_current_jackpot;
		$lastjackpot = $world->lotto_yesterday_jackpot;

		// if the minimum jackpot gets increased, apply it now
		if ($lastjackpot > $jackpot)
			$lastjackpot = LOTTERY_JACKPOT;

		$world->lotto_last_picked = $lotterynum;
		$world->lotto_yesterday_jackpot = $jackpot;
		$world->lotto_jackpot_increase = $jackpot - $lastjackpot;

		$win = $db->queryCell('SELECT e_id FROM '. LOTTERY_TABLE .' WHERE e_id > 0 AND l_ticket = ?', array($lotterynum));
		if ($win)
		{
			$emp_b = prom_empire::cached_load($win);
			addEmpireNews(EMPNEWS_ATTACH_LOTTERY, NULL, $emp_b, $jackpot);
			statecho('- Lottery results: ticket '. prenum($lotterynum) .' held by '. $emp_b .', '. money($jackpot) .' won.');
			$world->lotto_current_jackpot = LOTTERY_JACKPOT;
			$emp_b = NULL;
		}
		else
		{
			statecho('- Lottery results: ticket '. prenum($lotterynum) .' not held, no winner.');
			$win = 0;
		}
		$world->lotto_last_winner = $win;
		$db->query('DELETE FROM '. LOTTERY_TABLE);

		// Once per day, give points based on networth
		if (SCORE_ENABLE)
			$db->query('UPDATE '. EMPIRE_TABLE .' SET e_score = e_score + SQRT(e_networth / 1250000)');
	}

	statecho('Performing hourly events');
	// Various counters
	if (AID_ENABLE)
		$db->queryParam('UPDATE '. EMPIRE_TABLE .' SET e_aidcred = e_aidcred + 1 WHERE e_aidcred < ? AND u_id != 0', array(AID_MAXCREDITS));
	$db->query('UPDATE '. EMPIRE_TABLE .' SET e_vacation = e_vacation + 1 WHERE e_vacation > 0 AND u_id != 0');

	if (MAX_ATTACKS > 0)
	{
		$db->query('UPDATE '. EMPIRE_TABLE .' SET e_attacks = e_attacks + 1 WHERE e_attacks < 0 AND u_id != 0');
		$db->query('UPDATE '. EMPIRE_TABLE .' SET e_attacks = e_attacks + 1 WHERE e_attacks < 0 AND u_id != 0');
	}

	// the vacation check here MUST match the behavior in prom_empire::is_vacation_done()
	if (CUR_TIME >= $world->round_time_closing)
		$db->queryParam('UPDATE '. EMPIRE_TABLE .' SET e_vacation = 0, e_idle = ? WHERE e_vacation >= ? AND u_id != 0', array(CUR_TIME, VACATION_START + VACATION_LIMIT + 1));

	// clean up expired effects
	$db->queryParam('DELETE FROM '. EMPIRE_EFFECT_TABLE .' WHERE ef_name LIKE ? AND ef_value <= ?', array(EMPIRE_EFFECT_TIME .'%', CUR_TIME));
	$db->queryParam('DELETE FROM '. EMPIRE_EFFECT_TABLE .' WHERE ef_name LIKE ? AND ef_value <= 0', array(EMPIRE_EFFECT_TURN .'%'));
	$db->queryParam('DELETE FROM '. EMPIRE_EFFECT_TABLE .' WHERE ef_name LIKE ? AND ef_value = 0', array(EMPIRE_EFFECT_PERM .'%'));
}
statecho('Performing normal events');

// Give turns, overflowing into stored turns if necessary
$db->queryParam('UPDATE '. EMPIRE_TABLE .' SET e_turns = e_turns + ? + LEAST(e_storedturns, ?), e_storedturns = e_storedturns - LEAST(e_storedturns, ?) WHERE e_vacation = 0 AND e_flags & ? = 0 AND u_id != 0', array(TURNS_COUNT, TURNS_UNSTORE, TURNS_UNSTORE, EFLAG_DISABLE));
$db->queryParam('UPDATE '. EMPIRE_TABLE .' SET e_storedturns = LEAST(?, e_storedturns + e_turns - ?), e_turns = ? WHERE e_turns > ? AND u_id != 0', array(TURNS_STORED, TURNS_MAXIMUM, TURNS_MAXIMUM, TURNS_MAXIMUM));

// Reduce maximum private market sell percentage (by 1% base, up to 2% if the player has nothing but bldcash)
$db->query('UPDATE '. EMPIRE_TABLE .' SET e_mktperarm = e_mktperarm - LEAST(e_mktperarm, 100 * (1 + e_bldcash / e_land)) WHERE u_id != 0 AND e_land != 0');
$db->query('UPDATE '. EMPIRE_TABLE .' SET e_mktperlnd = e_mktperlnd - LEAST(e_mktperlnd, 100 * (1 + e_bldcash / e_land)) WHERE u_id != 0 AND e_land != 0');
$db->query('UPDATE '. EMPIRE_TABLE .' SET e_mktperfly = e_mktperfly - LEAST(e_mktperfly, 100 * (1 + e_bldcash / e_land)) WHERE u_id != 0 AND e_land != 0');
$db->query('UPDATE '. EMPIRE_TABLE .' SET e_mktpersea = e_mktpersea - LEAST(e_mktpersea, 100 * (1 + e_bldcash / e_land)) WHERE u_id != 0 AND e_land != 0');

// Refill private market based on bldcost (except for food, which uses bldfood)
$db->query('UPDATE '. EMPIRE_TABLE .' SET e_mktarm = e_mktarm + 8 * (e_land + e_bldcost) WHERE e_mktarm / 250 < e_land + 2 * e_bldcost AND u_id != 0');
$db->query('UPDATE '. EMPIRE_TABLE .' SET e_mktlnd = e_mktlnd + 5 * (e_land + e_bldcost) WHERE e_mktlnd / 200 < e_land + 2 * e_bldcost AND u_id != 0');
$db->query('UPDATE '. EMPIRE_TABLE .' SET e_mktfly = e_mktfly + 3 * (e_land + e_bldcost) WHERE e_mktfly / 180 < e_land + 2 * e_bldcost AND u_id != 0');
$db->query('UPDATE '. EMPIRE_TABLE .' SET e_mktsea = e_mktsea + 2 * (e_land + e_bldcost) WHERE e_mktsea / 150 < e_land + 2 * e_bldcost AND u_id != 0');
$db->query('UPDATE '. EMPIRE_TABLE .' SET e_mktfood = e_mktfood + 50 * (e_land + e_bldfood) WHERE e_mktfood / 2000 < e_land + 2 * e_bldfood AND u_id != 0');

// Various other counters
$db->queryParam('UPDATE '. EMPIRE_TABLE .' SET e_msgcred = e_msgcred + 1 WHERE e_msgcred < ? AND u_id != 0', array(MESSAGES_MAXCREDITS));
if (MAX_ATTACKS > 0)
	$db->query('UPDATE '. EMPIRE_TABLE .' SET e_attacks = e_attacks - 1 WHERE e_attacks > 0 AND u_id != 0');
if (CLAN_ENABLE)
	$db->query('UPDATE '. EMPIRE_TABLE .' SET e_sharing = e_sharing - 1 WHERE e_sharing > 0 AND u_id != 0');

// clean up expired clan invites
if (CLAN_ENABLE)
	$db->queryParam('DELETE FROM '. CLAN_INVITE_TABLE .' WHERE ci_flags & ? = 0 AND ci_time <= ?', array(CIFLAG_PERM, CUR_TIME + CLAN_INVITE_TIME * 3600));

if (PUBMKT_MAXTIME >= 0)
{
	statecho('Cleaning market');

	$q = $db->queryParam('SELECT * FROM '. MARKET_TABLE .' WHERE k_time <= ?', array(CUR_TIME - 3600 * PUBMKT_MAXTIME));
	$markets = $q->fetchAll();
	$defcost = lookup('pubmkt_id_cost');
	foreach ($markets as $market)	// send them back
	{
		$emp_b = prom_empire::cached_load($market['e_id']);
		$basecost = $defcost[$market['k_type']];
		$lost = floor($market['k_amt'] * (min(max($market['k_price'] - $basecost, 0) / $basecost, 0.3) + 0.2));
		addEmpireNews(EMPNEWS_ATTACH_MARKET_RETURN, NULL, $emp_b, $market['k_type'], $market['k_amt'], $market['k_price'], $market['k_amt'] - $lost);
		$emp_b = NULL;
		// lost goods fund the jackpot at 20% of their base value
		$world->lotto_current_jackpot += round($lost * $basecost / 5);
	}
	$db->queryParam('DELETE FROM '. MARKET_TABLE .' WHERE k_time <= ?', array(CUR_TIME - 3600 * PUBMKT_MAXTIME));	// and delete them
}

statecho('Unlinking empires');
// the protection check here MUST match the behavior in prom_empire::is_protected()
$q = $db->queryParam('SELECT e_id FROM '. EMPIRE_TABLE .' WHERE u_id != 0 AND e_flags & ? = 0 AND ('.
	'(e_flags & ? = ? AND e_idle < ?) OR '.
	'(e_flags & ? = ? AND e_idle < ?) OR '.
	'(e_flags & ? = 0 AND e_vacation = 0 AND e_land > 0 AND e_idle < ?) OR '.
	'(e_land = 0 AND (e_flags & ? = ? OR e_idle < ?)) OR '.
	'(e_flags & ? = ? AND (e_turnsused <= ? OR e_idle < ?))'.
		')', array(EFLAG_ADMIN,						// don't ever delete admins
	EFLAG_VALID | EFLAG_NOTIFY, 0, CUR_TIME - 86400 * IDLE_TIMEOUT_NEW,			// not validated or notified
	EFLAG_VALID | EFLAG_NOTIFY, EFLAG_NOTIFY, CUR_TIME - 86400 * IDLE_TIMEOUT_VALIDATE,	// notified but not validated
	EFLAG_DISABLE, CUR_TIME - 86400 * IDLE_TIMEOUT_ABANDON, 				// not disabled or on vacation
	EFLAG_NOTIFY, EFLAG_NOTIFY, CUR_TIME - 86400 * IDLE_TIMEOUT_KILLED,			// dead (immediate if notified)
	EFLAG_DELETE, EFLAG_DELETE, TURNS_PROTECTION, CUR_TIME - 86400 * IDLE_TIMEOUT_DELETE,	// marked for deletion (immediate if still under protection)
		));
$emps = $q->fetchAll();

// shortcut for inheriting clan leadership from another member, since we need to attempt this up to 3 times per empire
function inheritClan ($emp_a, $clan_a, $emp_b)
{
	$event = CLANNEWS_PERM_MEMBER_INHERIT;
	$clan_a->e_id_leader = $emp_b->e_id;
	if ($clan_a->e_id_fa1 == $emp_b->e_id)
	{
		$clan_a->e_id_fa1 = 0;
		$event = CLANNEWS_PERM_MINISTER_INHERIT;
	}
	if ($clan_a->e_id_fa2 == $emp_b->e_id)
	{
		$clan_a->e_id_fa2 = 0;
		$event = CLANNEWS_PERM_MINISTER_INHERIT;
	}

	addEmpireNews(EMPNEWS_CLAN_INHERIT_LEADER, $emp_a, $emp_b, 0);
	addClanNews($event, $clan_a, $emp_b, NULL, $emp_a);
}

foreach ($emps as $row)
{
	$emp_a = new prom_empire($row['e_id'], FALSE); // do not require lock
	$emp_a->lock(LOCK_TURNS);
	statecho('- Unlinking empire '. $emp_a);
	$reason = '';
	if (CLAN_ENABLE && $emp_a->c_id)
	{	// remove user from clan
		$clan_a = new prom_clan($emp_a->c_id, FALSE); // do not require lock
		$clan_a->lock(LOCK_TURNS);
		if ($clan_a->e_id_leader == $emp_a->e_id)
		{	// transfer ownership if possible
			$reason .= 'clan leader, ';
			// first candidate is the assistant leader, alive or not - if he's dead, it'll transfer again later
			if ($clan_a->e_id_asst)
			{
				$clan_a->e_id_leader = $clan_a->e_id_asst;
				$clan_a->e_id_asst = 0;

				$emp_b = prom_empire::cached_load($clan_a->e_id_leader);
				addEmpireNews(EMPNEWS_CLAN_INHERIT_LEADER, $emp_a, $emp_b, 0);
				addClanNews(CLANNEWS_PERM_ASSISTANT_INHERIT, $clan_a, $emp_b, NULL, $emp_a);
				$reason .= 'to asst '. $emp_b .', ';
				$emp_b = NULL;
			}
			// next, try to find the strongest member who isn't dead, deleted, or disabled
			elseif ($found = $db->queryCell('SELECT e_id FROM '. EMPIRE_TABLE .' WHERE c_id = ? AND e_id != ? AND e_flags & ? = 0 AND e_land > 0 ORDER BY e_networth DESC', array($clan_a->c_id, $emp_a->e_id, EFLAG_DELETE | EFLAG_DISABLE)))
			{
				$emp_b = prom_empire::cached_load($found);
				inheritClan($emp_a, $clan_a, $emp_b);
				$reason .= 'to member '. $emp_b .', ';
				$emp_b = NULL;
			}
			// if everybody's dead, then pick one of them anyways (as long as they aren't disabled)
			elseif ($found = $db->queryCell('SELECT e_id FROM '. EMPIRE_TABLE .' WHERE c_id = ? AND e_id != ? AND e_flags & ? = 0 ORDER BY e_networth DESC', array($clan_a->c_id, $emp_a->e_id, EFLAG_DISABLE)))
			{
				$emp_b = prom_empire::cached_load($found);
				inheritClan($emp_a, $clan_a, $emp_b);
				$reason .= 'to member '. $emp_b .' (also dead), ';
				$emp_b = NULL;
			}
			// last resort, NEED to choose a leader, even if it's disabled
			elseif ($found = $db->queryCell('SELECT e_id FROM '. EMPIRE_TABLE .' WHERE c_id = ? AND e_id != ? ORDER BY e_networth DESC', array($clan_a->c_id, $emp_a->e_id)))
			{
				$emp_b = prom_empire::cached_load($found);
				inheritClan($emp_a, $clan_a, $emp_b);
				$reason .= 'to member '. $emp_b .' (DISABLED), ';
				$emp_b = NULL;
			}
			// by now, said empire is guaranteed to be the last member, so the clan will be deleted in a moment
			else
			{
				$clan_a->e_id_leader = 0;
				$reason .= 'abandoned, ';
			}
		}
		elseif ($clan_a->e_id_asst == $emp_a->e_id)
		{
			$clan_a->e_id_asst = 0;
			$reason .= 'clan asst, ';
		}
		elseif ($clan_a->e_id_fa1 == $emp_a->e_id)
		{
			$clan_a->e_id_fa1 = 0;
			$reason .= 'clan fa1, ';
		}
		elseif ($clan_a->e_id_fa2 == $emp_a->e_id)
		{
			$clan_a->e_id_fa2 = 0;
			$reason .= 'clan fa2, ';
		}
		$clan_a->c_members--;
		$clan_a->save();
		$clan_a = NULL;

		$emp_a->c_oldid = $emp_a->c_id;
		$emp_a->c_id = 0;		// remove from clan
	}
	// delete any messages to/from that user
	$db->queryParam('UPDATE '. EMPIRE_MESSAGE_TABLE .' SET m_flags = m_flags | ? WHERE e_id_src = ? OR e_id_dst = ?', array(MFLAG_DELETE, $emp_a->e_id, $emp_a->e_id));
	// any of the user's items on the market
	$db->queryParam('DELETE FROM '. MARKET_TABLE .' WHERE e_id = ?', array($emp_a->e_id));
	// any lottery tickets
	$db->queryParam('DELETE FROM '. LOTTERY_TABLE .' WHERE e_id = ?', array($emp_a->e_id));

	if ($emp_a->e_flags & EFLAG_DELETE)
	{
		if ($emp_a->e_killedby == $emp_a->e_id)
		{
			$reason .= 'deleted';
			if ($emp_a->e_reason)
				$emp_a->e_reason = 'Self-deleted: '. $emp_a->e_reason;
			else	$emp_a->e_reason = 'Self-deleted (no reason)';
		}
		else
		{
			$reason .= 'nuked';
			if ($emp_a->e_reason)
				$emp_a->e_reason = 'Deleted by admin: '. $emp_a->e_reason;
			else	$emp_a->e_reason = 'Deleted by admin (no reason)';
		}
	}
	elseif (($emp_a->e_flags & (EFLAG_VALID | EFLAG_NOTIFY)) == 0)
	{
		$reason .= 'ignored';
		$emp_a->e_reason = 'Auto-deleted, never used enough turns to require validation';
		$emp_a->e_killedby = 0;
	}
	elseif (($emp_a->e_flags & (EFLAG_VALID | EFLAG_NOTIFY)) == EFLAG_NOTIFY)
	{
		$reason .= 'unvalidated';
		$emp_a->e_reason = 'Auto-deleted, failed to validate';
		$emp_a->e_killedby = 0;
	}
	elseif ($emp_a->e_land == 0)
	{
		$reason .= 'killed';
		$emp_a->e_reason = 'Killed';
	}
	elseif ((($emp_a->e_flags & EFLAG_DISABLE) == 0) && ($emp_a->e_vacation == 0))
	{
		$reason .= 'abandoned';
		$emp_a->e_reason = 'Auto-deleted, left idle for an excessive duration';
		$emp_a->e_killedby = 0;
	}
	else
	{
		$reason .= 'indeterminate';
		$emp_a->e_reason = 'Unknown deletion reason';
		$emp_a->e_killedby = 0;
	}

	$emp_a->u_oldid = $emp_a->u_id;
	$emp_a->u_id = 0;		// unlink from user
	$emp_a->save();
	statecho('- Empire '. $emp_a .' unlinked - '. $reason);
	$emp_a = NULL;
}

if (CLAN_ENABLE)
{
	statecho('Removing clans');
	// need a dummy record here so we can record the clan of origin
	$emp_a = new prom_news_placeholder();
	$q = $db->query('SELECT c_id FROM '. CLAN_TABLE .' WHERE c_members = 0');
	$clans = $q->fetchAll();
	foreach ($clans as $row)
	{	// remove all associations with empty clans and make clan invisible to game
		$clan_a = new prom_clan($row['c_id'], FALSE);
		$clan_a->lock(LOCK_TURNS);
		statecho('- Removing clan '. $clan_a);
		// can't actually delete the row, since news table entries will still refer to it
		$clan_a->c_members = -1;
		$emp_a->c_id = $clan_a->c_id;	// assign clan ID to dummy record

		$q1 = $db->queryParam('SELECT e_id_leader AS e_id,cr_flags,c_id FROM '. CLAN_RELATION_TABLE .' LEFT OUTER JOIN '. CLAN_TABLE .' ON (c_id_2 = c_id) WHERE c_id_1 = ?', array($clan_a->c_id));
		$q2 = $db->queryParam('SELECT e_id_leader AS e_id,cr_flags,c_id FROM '. CLAN_RELATION_TABLE .' LEFT OUTER JOIN '. CLAN_TABLE .' ON (c_id_1 = c_id) WHERE c_id_2 = ?', array($clan_a->c_id));
		$rels = array_merge($q1->fetchAll(), $q2->fetchAll());
		foreach ($rels as $rel)
		{
			// if two clans are disbanded simultaneously and they had relations with each other, skip the news reports
			if ($rel['e_id'] == 0)
				continue;
			$emp_b = prom_empire::cached_load($rel['e_id']);
			$clan_b = new prom_news_placeholder(0, $rel['c_id']);
			if ($rel['cr_flags'] & CRFLAG_ALLY)
			{
				$empevent = EMPNEWS_CLAN_ALLY_GONE;
				$clanevent = CLANNEWS_RECV_ALLY_GONE;
			}
			else
			{
				$empevent = EMPNEWS_CLAN_WAR_GONE;
				$clanevent = CLANNEWS_RECV_WAR_GONE;
			}
			addEmpireNews($empevent, $emp_a, $emp_b, 0);
			addClanNews($clanevent, $clan_b, NULL, $clan_a);
			$clan_b = NULL;
			$emp_b = NULL;
		}
		$db->queryParam('DELETE FROM '. CLAN_RELATION_TABLE .' WHERE c_id_1 = ? OR c_id_2 = ?', array($clan_a->c_id, $clan_a->c_id));
		$clan_a->save();
		statecho('- Clan '. $clan_a .' removed.');
		$clan_a = NULL;
	}
	$emp_a = NULL;
}

statecho('Updating ranks');
$db->query('UPDATE '. EMPIRE_TABLE .' SET e_rank = 0 WHERE u_id = 0');
// If score tracking is enabled, rank players by their score instead of their networth
if (SCORE_ENABLE)
	$q = $db->query('SELECT e_id FROM '. EMPIRE_TABLE .' WHERE u_id != 0 ORDER BY e_score DESC, e_networth DESC');
else	$q = $db->query('SELECT e_id FROM '. EMPIRE_TABLE .' WHERE u_id != 0 ORDER BY e_networth DESC');
$emps = $q->fetchAll();
$urank = 0;
foreach ($emps as $emp)
{
	$urank++;
	$db->queryParam('UPDATE '. EMPIRE_TABLE .' SET e_rank = ? WHERE e_id = ?', array($urank, $emp['e_id']));
}

// check to see if it's okay to end the round early
if (CUR_TIME >= $world->round_time_closing)
{
	$numplayers = $db->queryCell('SELECT COUNT(*) FROM '. EMPIRE_TABLE .' WHERE u_id != 0 AND e_flags & ? = 0', array(EFLAG_ADMIN));
	if ($numplayers == 1)
	{
		statecho('Only one survivor remains - ending round');
		$world->round_time_end = CUR_TIME;
	}
}

statecho('Flushing sessions');
// after 3 turns updates of being idle (or if they've been unlinked), set empires as offline
$db->queryParam('UPDATE '. EMPIRE_TABLE .' SET e_flags = e_flags & ? WHERE e_idle < ? OR u_id = 0', array(~EFLAG_ONLINE, CUR_TIME - 3 * 60 * TURNS_FREQ));

unlock_all();

done();
?>
